org 100h       ; assume cx=0xff ah=0 si=0x100 bl=0

  push 0a000h
  pop es

; pseudorandom initialization (from program bytes)
init:          ; cx=0xff si=100h
  lodsb
  and al,0xF   ; we want size to be <= 20
  mov [byte si-100h+vars-1],al
  loop init    ; cx=0

  cwd          ; dx=0
  mov al,13h   ; set video mode
pal:
  int 10h
  mov ax,1010h ; set palette index: bl=index, dh=R=index*6 ch=G=0 cl=B=0
  sub dh,6
  dec bl       ; pal index
  jnz pal      ; dx=0

main:
  mov si,vars
  mov bl,9     ; 9 squares

square_loop:
  mov cl,20    ; y: 20..1

  lodsw
  imul di,ax,85
  inc di       ; di = next address of top left pixel (LCG pseudorandom generator)

  dec byte[si] ; decrease size: if -1, reset position and size
  jns skipreset
  mov [si-2],di; next position = pseudorandom
  mov [si],cl  ; next size = 20
skipreset:

  lodsb        ; al=size
  sub al,10    ; 0..10..20 -> -10..0..10 (saw to triangle)
  js already_neg
  neg al       ; -10..0..-10
already_neg:
  add al,10    ; 0..10..0

; draw a 20x20 black square with (2*AL)x(2*AL) colored square in the center
y:
  mov ch,20    ; x: 20..1
x:

  pusha        ; we only really need to push ax and cx
hv:            ; if |y-10|>size or |x-10|>size, draw black
  sub cl,11
  jns positive ; 1..10,11..20 -> -10..-1,0..9 -> 9..0,0..9
  not cl
positive:
  cmp cl,al
  ja black

  shr cx,8     ; now test ch
  jnz hv

  db 0x3c ; = jmp colored, but only 1 byte
black:
  salc   ; al=0 (carry flag = 0 when we get here)
colored:
  stosb
  popa

  inc di
  dec ch
  jnz x

  add di,320-20
  loop y

  mov dx,3dah ; wait for retrace after each square
wait_end:
  in al,dx
  test al,8
  jnz wait_end
wait_start:
  in al,dx
  test al,8
  jz wait_start

  dec bx
  jnz square_loop

  in al,60h ; wait for ESC after drawing all 9 squares
  dec al
  jne main
  ret

vars: ; 9 * [dw adr, db size]
